#define DEFAULT_MODE 0
#define BOMB_MODE 1
#define MAX_MODE 2
#define BREAK_MODE 3
const int ledPin1 = 10;
const int hallPin = 2;
const int ledPin2 = 11;
const int ledPin3 = 12;
const int buzzerPin = 13;
const int resetButtonPin = 3;
const int speedButtonPin = 9;
const int confirmButtonPin = 8;
int rotations;
int mass;
char isConfirmed;
float distance;
float frictionForce;
float rotSpeed;
float nrLeds;
unsigned long startTime;
int mode = 0;
int ledArray[3];
volatile bool ledState[3];
int timerDone;
void setup() 
{
  pinMode(ledPin1,OUTPUT);
  pinMode(ledPin2,OUTPUT);
  pinMode(ledPin3,OUTPUT);
  pinMode(hallPin,INPUT);
  pinMode(resetButtonPin,INPUT_PULLUP);
  pinMode(speedButtonPin,INPUT_PULLUP);
  pinMode(confirmButtonPin,INPUT_PULLUP);
  pinMode(buzzerPin,OUTPUT);
  Serial.begin(9600);
  attachInterrupt(digitalPinToInterrupt(hallPin), hall_ISR, CHANGE);
  attachInterrupt(digitalPinToInterrupt(resetButtonPin), mode_ISR, FALLING);
  rotations = 0;
  mass = 80;
  frictionForce = 0.75 * mass * 9.81;
  distance = 0;
  
  ledArray[0] = ledPin1;
  ledArray[1] = ledPin2;
  ledArray[2] = ledPin3;

  ledState[0] = LOW;
  ledState[1] = LOW;
  ledState[2] = LOW;

  Serial.println("Default mode");
  
  
}
void checkMinSpeed()
{
  unsigned long lastIncDbTime = millis();
  unsigned long debounceDelay = 500;
  nrLeds = 0;
  isConfirmed = 0;
  while(isConfirmed==0)
  {
    int currentState = digitalRead(speedButtonPin);
    if((millis() - lastIncDbTime) > debounceDelay)
    {
      lastIncDbTime = millis();
      if(currentState == LOW && nrLeds <= 2)
      {
        nrLeds++;
        for(int i=0;i<nrLeds;i++)
        {
          ledState[i] = HIGH;
          digitalWrite(ledArray[i], ledState[i]);
        }
        Serial.print("You have set minnimum speed to ");
        Serial.print(nrLeds,DEC);
        Serial.println(" m/s");
      }
    }
    if(digitalRead(confirmButtonPin) == LOW)
    {
      isConfirmed = 1;
      Serial.println("Speed confirmed");
    }
    
  }
}

void bombMode()
{
  Serial.println("Welcome to bomb mode. Please set minimum training speed");
  rotations = 0;
  checkMinSpeed();
  startTime = millis(); 
  while(mode == BOMB_MODE)
  {
    
    distance = 3.1415 * 0.66 * rotations;
    rotSpeed = distance*1000/(millis() - startTime);
    if(rotSpeed > 0){
      Serial.print("Speed: ");
      Serial.println(rotSpeed,DEC);

      if(rotSpeed < nrLeds)
      {
        tone(buzzerPin,1000);
      }
      else
      {
        noTone(buzzerPin);
      }
    }
    
    
  }

  for(int i=0;i<3;i++){
    ledState[i] = LOW;
    digitalWrite(ledArray[i],ledState[i]);
  }

  
  nrLeds = 0;
  noTone(buzzerPin);
}

ISR(TIMER1_COMPA_vect) 
{
  Serial.println("Time is up!");
  timerDone = 1;
}

void maxSpeedMode()
{
  Serial.println("Max Speed mode activated...");
  unsigned long startTime = millis();
  rotations = 0;
  detachInterrupt(digitalPinToInterrupt(resetButtonPin));
  while(millis() - startTime < 10000)
  {
    distance = 3.1415 * 0.66 * rotations;
    Serial.print("Distance: ");
    Serial.println(distance, DEC);
  }
  float finalSpeed = distance*1000/(millis() - startTime);
  float totalWork = (mass / 2.0) * finalSpeed * finalSpeed + frictionForce * distance;
  Serial.println(totalWork, DEC);
  delay(7000);
  attachInterrupt(digitalPinToInterrupt(resetButtonPin),mode_ISR,FALLING);
  
}
void breakMode()
{
  Serial.println("Break mode");

  while(mode == BREAK_MODE)
  {
    if(digitalRead(speedButtonPin) == LOW)
    {
      detachInterrupt(digitalPinToInterrupt(resetButtonPin));
      for(int i=2;i>=0;i--)
      {
        digitalWrite(ledArray[i],HIGH);
        delay(100);
        digitalWrite(ledArray[i],LOW);
      }
      delay(500);
      attachInterrupt(digitalPinToInterrupt(resetButtonPin),mode_ISR,FALLING);
    }
    else if(digitalRead(confirmButtonPin) == LOW)
    {
      detachInterrupt(digitalPinToInterrupt(resetButtonPin));
      for(int i=0;i<=2;i++)
      {
        digitalWrite(ledArray[i],HIGH);
        delay(100);
        digitalWrite(ledArray[i],LOW);
      }
      delay(500);
      attachInterrupt(digitalPinToInterrupt(resetButtonPin),mode_ISR,FALLING);
    }
  }
}

void loop() 
{
  switch(mode)
  {
    case DEFAULT_MODE:
      delay(500);
      break;

    case BOMB_MODE:
      bombMode();
      break;

    case MAX_MODE:
      maxSpeedMode();
      break;

    case BREAK_MODE:
      breakMode();
      break;
      

      
  }


  
}


void hall_ISR() 
{
rotations++;
}

void mode_ISR()
{
  mode = (mode + 1)%4;
  if(mode == DEFAULT_MODE)
    Serial.println("Default mode");
}
